@@ -7,6 +7,7 @@ import warnings
 import fitsio
 import numpy as np
 
+from qsonic import QsonicException
 import qsonic.spectrum
 
 
@@ -30,7 +31,7 @@ def add_io_parser(parser=None):
     ingroup = parser.add_argument_group('Input options')
     ingroup.add_argument(
         "--input-dir", '-i', required=True,
-        help="Input directory to healpix")
+        help="Input directory.")
     ingroup.add_argument(
         "--catalog", required=True,
         help="Catalog filename")
@@ -133,6 +134,34 @@ def read_spectra_onehealpix(
     return spectra_list
 
 
+def read_deltas(fname):
+    """ Returns a list of all Delta objects in a file.
+
+    Arguments
+    ---------
+    fname: str
+        FITS file name
+
+    Returns
+    ---------
+    deltas_list: list(Delta)
+
+    Raises
+    ---------
+    RuntimeError
+        If the file is missing certain columns and keys. See :class:`Delta`.
+    """
+    deltas_list = []
+    fitsfile = fitsio.FITS(fname)
+
+    for hdu in fitsfile[1:]:
+        deltas_list.append(qsonic.spectrum.Delta(hdu))
+
+    fitsfile.close()
+
+    return deltas_list
+
+
 def save_deltas(
         spectra_list, outdir, varlss_interp,
         save_by_hpx=False, mpi_rank=None, use_ivar_sm=False
@@ -157,9 +186,9 @@ def save_deltas(
 
     Raises
     ---------
-    Exception
+    QsonicException
         If both `mpi_rank` and `save_by_hpx` is None/False.
-    Exception
+    QsonicException
         If blinding is not set.
     """
     if not outdir:
@@ -175,10 +204,10 @@ def save_deltas(
         unique_pix = [mpi_rank]
         split_spectra = [spectra_list]
     else:
-        raise Exception("save_by_hpx and mpi_rank can't both be None.")
+        raise QsonicException("save_by_hpx and mpi_rank can't both be None.")
 
     if qsonic.spectrum.Spectrum.blinding_not_set():
-        raise Exception("Blinding is not set. Cannot save deltas.")
+        raise QsonicException("Blinding is not set. Cannot save delta.")
 
     for healpix, hp_specs in zip(unique_pix, split_spectra):
         results = fitsio.FITS(
@@ -190,21 +219,22 @@ def save_deltas(
         results.close()
 
 
-def _read_resoimage(imhdu, quasar_indices, sort_idx, nwave):
+def _read_resoimage(imhdu, quasar_indices, nwave):
     # Reading into sorted order is faster
+    # Since the output catalog is already sorted, we place them in order
     ndiags = imhdu.get_dims()[1]
     dtype = imhdu._get_image_numpy_dtype()
     data = np.empty((quasar_indices.size, ndiags, nwave), dtype=dtype)
-    for idata, iqso in zip(sort_idx, quasar_indices):
+    for idata, iqso in enumerate(quasar_indices):
         data[idata, :, :] = imhdu[int(iqso), :, :]
 
     return data
 
 
-def _read_imagehdu(imhdu, quasar_indices, sort_idx, nwave):
+def _read_imagehdu(imhdu, quasar_indices, nwave):
     dtype = imhdu._get_image_numpy_dtype()
     data = np.empty((quasar_indices.size, nwave), dtype=dtype)
-    for idata, iqso in zip(sort_idx, quasar_indices):
+    for idata, iqso in enumerate(quasar_indices):
         data[idata, :] = imhdu[int(iqso), :]
 
     return data
@@ -261,7 +291,6 @@ def _read_onehealpix_file(
             f"healpix:{common_targetids.size}!", RuntimeWarning)
 
     fbrmap = fbrmap[idx_fbr]
-    sort_idx = np.argsort(idx_fbr)
 
     data = {
         'wave': {},
@@ -277,17 +306,17 @@ def _read_onehealpix_file(
         nwave = data['wave'][arm].size
 
         data['flux'][arm] = _read_imagehdu(
-            fitsfile[f'{arm}_FLUX'], idx_fbr, sort_idx, nwave)
+            fitsfile[f'{arm}_FLUX'], idx_fbr, nwave)
         data['ivar'][arm] = _read_imagehdu(
-            fitsfile[f'{arm}_IVAR'], idx_fbr, sort_idx, nwave)
+            fitsfile[f'{arm}_IVAR'], idx_fbr, nwave)
         data['mask'][arm] = _read_imagehdu(
-            fitsfile[f'{arm}_MASK'], idx_fbr, sort_idx, nwave)
+            fitsfile[f'{arm}_MASK'], idx_fbr, nwave)
 
         if skip_resomat or f'{arm}_RESOLUTION' not in fitsfile:
             continue
 
         data['reso'][arm] = _read_resoimage(
-            fitsfile[f'{arm}_RESOLUTION'], idx_fbr, sort_idx, nwave)
+            fitsfile[f'{arm}_RESOLUTION'], idx_fbr, nwave)
 
     fitsfile.close()
 
